import pathlib
from distutils.core import setup
import setuptools
HERE=pathlib.Path(__file__).parent
README=(HERE/"README.md").read_text()

setup(
  name = 'face-mask-classifier',         # How you named your package folder (MyLib)
  packages = ['face-mask-classifier/src','face-mask-classifier/models'],   # Chose the same as "name"
  includepackagedata=True,
  version = '0.6',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Detect face mask in an Image',# Give a short description about your library
  long_descp=README,
  long_descp_content="text/markdown",  
  author = 'Baskaran Thulukanam',                   # Type in your name
  author_email = 'baskar.mailbox@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/Baskar-t/face-mask-classifier',   # Provide either the link to your github or to your website
  #download_url='https://github.com/Baskar-t/face-mask-classifier/archive/refs/tags/0.4.tar.gz',
  keywords = ['face-mask-classifer', 'SVM-classifier'],   # Keywords that define your package best
  Install_requires=[            # I get to this in a second
          'Keras==2.4.3',
          'keras-facenet==0.3.2',
          'tensorflow==2.5.0',
          'scikit-learn==0.24.2',
          'mtcnn==0.1.0',
          'numpy==1.19.5'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)


import numpy as np
import argparse
import cv2
import os
from keras_facenet import FaceNet
import pickle


class mask_predictor():
  def __init__(self):
    pass

  def pred_mask(image,svm_model_path):
    img=cv2.imread(image)
    #load facenet models
    print("[INFO] loading face detector model...")
    face_detect_model=FaceNet()
    #load mask classifier models
    print("[INFO] loading face mask detector model...")
    model = pickle.load(open(svm_model_path, 'rb'))
    faces_array_train = face_detect_model.extract(image, threshold=0.95)
    embed_list_test=[]
    face_box_list=[]
    face_confidence_list=[]
    for j in range(len(faces_array_train)): 
        face_embed_dict=faces_array_train[j]
        face_embed=face_embed_dict['embedding']
        face_confidence=face_embed_dict['confidence']
        face_box=face_embed_dict['box']
        embed_list_test.append(face_embed)
        face_box_list.append(face_box)
        face_confidence_list.append(face_confidence)
    
    face=np.array(embed_list_test)
    k=0
    label_list=[]
    proba_list=[]
    box_list=[]
    for i in face:
      i=i.reshape(1,-1)
      label_pred = model.predict(i)
      proba = model.predict_proba(i)
      if label_pred == 0:
          label = "Mask"
      else:
          label= "No Mask"
      if label=='Mask':
        proba= proba[0][0]
      else:
        proba=proba[0][1]
      box=face_box_list[k]
      box_list.append(box)
      label_list.append(label)
      proba_list.append(proba)
      k=k+1

    return img,box_list,label_list,proba_list


  




import tensorflow as tf
import pandas as pd
from numpy import save
import tqdm
import os
from PIL import Image
import numpy as np
import cv2
from keras_facenet import FaceNet
import pandas as pd 
from keras.utils.np_utils import to_categorical 
from sklearn.preprocessing import LabelBinarizer 
from keras.models import load_model
from sklearn.preprocessing import Normalizer
from sklearn.preprocessing import LabelEncoder
from sklearn.svm import SVC
from sklearn.metrics import accuracy_score
import pickle
from numpy import load

class train_classifier():
  def __init__(self):
    model=FaceNet()
  
  def processed_data(train_data_path,validation_data_path):
    model=FaceNet()
    # Load the train images
    df_train = pd.read_csv('/d/project2/Input/train.csv')
    filenames_train=df_train[df_train.columns[0]]
    labels_train=df_train[df_train.columns[1]]

    embed_list_train=[]
    label_train_list=[]
    k=0
    for i in tqdm.tqdm(filenames_train):    
        faces_array_train = model.extract(i, threshold=0.95)
        for j in range(len(faces_array_train)):
            face_embed_dict=faces_array_train[j]
            face_embed=face_embed_dict['embedding']
            embed_list_train.append(face_embed)
            label_train_list.append(labels_train[k])
        k=k+1

    embed_list_train=np.array(embed_list_train)
    label_train_list=np.array(label_train_list)

    lb = LabelBinarizer()
    label_train_list = lb.fit_transform(label_train_list)
    label_train_list = to_categorical(label_train_list)

    df_val = pd.read_csv('/d/project2/Input/val.csv')
    filenames_val=df_val[df_val.columns[0]]
    labels_val=df_val[df_val.columns[1]]
    embed_list_val=[]
    label_val_list=[]
    k=0
    for i in tqdm.tqdm(filenames_val):    
        faces_array_val = model.extract(i, threshold=0.95)
        for j in range(len(faces_array_val)):
            face_embed_dict=faces_array_val[j]
            face_embed=face_embed_dict['embedding']
            embed_list_val.append(face_embed)
            label_val_list.append(labels_val[k])
        k=k+1

    embed_list_val=np.array(embed_list_val)
    label_val_list=np.array(label_val_list)

    lb = LabelBinarizer()
    label_val_list = lb.fit_transform(label_val_list)
    label_val_list = to_categorical(label_val_list)

    np.savez('svm_processed_data.npz', embed_list_train, label_train_list, embed_list_val, label_val_list)


  def svm_train(data_path):
    # load dataset
    data = load(data_path)
    trainX, trainy, testX, testy = data['arr_0'], data['arr_1'], data['arr_2'], data['arr_3']
    # normalize input vectors
    in_encoder = Normalizer(norm='l2')
    trainX = in_encoder.transform(trainX)
    testX = in_encoder.transform(testX)
    # label encode targets
    trainy = np.argmax(trainy, axis=1)
    testy = np.argmax(testy, axis=1)
    # fit model
    model = SVC(kernel='linear', probability=True)
    model.fit(trainX, trainy)
    # predict
    yhat_train = model.predict(trainX)
    yhat_test = model.predict(testX)
    # score
    score_train = accuracy_score(trainy, yhat_train)
    score_test = accuracy_score(testy, yhat_test)
    # summarize
    print('Accuracy: train=%.3f, test=%.3f' % (score_train*100, score_test*100))

    # save the model to disk
    filename = 'svm_classifier_model_initial.sav'
    pickle.dump(model, open(filename, 'wb'))


